function ExternalApplicationButtons()
{  
  this._init();
}

ExternalApplicationButtons.prototype={

  _wm: null,
  _current:null,

  set currentElement(aElement){
    if(this._current){
	if(this._current.database)
		this._current.database.RemoveDataSource(this._ds);
	this._current.ref="";
	if(this._current != aElement && this._current.builder)
		this._current.builder.rebuild();
    }

    if(aElement){
      aElement.database.AddDataSource(this._ds);
      aElement.ref=this._ref;
      aElement.builder.rebuild();
    }
    this._current=aElement;
  },

  get _uri(){
    const directoryService=Components.classes['@mozilla.org/file/directory_service;1']
      .getService(Components.interfaces.nsIProperties);
    var file = directoryService.get('ProfD',
                                      Components.interfaces.nsILocalFile);
    file.append("externalapp.rdf");
    if(!file.exists())
      file.create(file.NORMAL_FILE_TYPE,0666);
    return this.getURLFromFile(file);
  },

  getURLFromFile:function(aFile){
    const ioService=Components.classes['@mozilla.org/network/io-service;1']
      .getService(Components.interfaces.nsIIOService); 
    return ioService.newFileURI(aFile).spec;
  },

  get _ref()
  {
    return "urn:externalapp:buttons";
  },

  settings : function()
  {
    window.open("chrome://externalapp/content/settings.xul","_blank","chrome,modal");
  },

  properties : function(name)
  {
    window.open("chrome://externalapp/content/properties.xul",name,"chrome,modal");
  },

  get _ns(){
    return "http://forums.mozillazine.org/rdf/externalapp#";
  },

  _arc:function(aName){
    return this._rdfService.GetResource(this._ns + aName);
  },


  _getArguments :function(aStr){
    var arguments=[];
    var addressbar  =this.address;
    var selectedtext=this.selected;
    var clipboard  =this.clipboard;

    var argumentstext=aStr.split(/\s/);

    while(argumentstext.length  > 0){
      var text= argumentstext.shift();
      text=text.replace(/\\\\/g , "%backslash%");
      text=text.replace(/\\s/g , " ");
      text=text.replace(/\\a/g , "\a");
      text=text.replace(/\\e/g , "\e");
      text=text.replace(/\\f/g , "\f");
      text=text.replace(/\\n/g , "\n");
      text=text.replace(/\\r/g , "\r");
      text=text.replace(/\\t/g , "\t");
      text=text.replace(/\\v/g , "\v");
      text=text.replace(/\\/g , "");

      text=text.replace(/%backslash%/g , "\\");
      text=text.replace(/%addressbar%/g , addressbar);
      text=text.replace(/%selectedtext%/g , selectedtext);
      text=text.replace(/%clipboard%/g , clipboard);
      if(text !="") arguments.push(text);
    }
    return arguments;
  },

  get _FFDir(){
    const directoryService=Components.classes['@mozilla.org/file/directory_service;1']
      .getService(Components.interfaces.nsIProperties);
    var fileOrDir = directoryService.get('CurProcD', Components.interfaces.nsIFile);
    var FFdir = fileOrDir.QueryInterface(Components.interfaces.nsILocalFile).path;
    FFdir += "\\"
    return FFdir
  },

  _launch :function(path,aArg){
    try{
      if(!path)
        return false;

      if(/^(\.)/.test(path)){
	path = this._FFDir.concat(path);
      }
      var file = Components.classes['@mozilla.org/file/local;1']
              .createInstance(Components.interfaces.nsILocalFile);
      var process = Components.classes['@mozilla.org/process/util;1']
              .getService(Components.interfaces.nsIProcess);
      file.initWithPath(path);

      if(! file.exists())
        return false;
      if(! file.isExecutable() || path.match(/\.lnk$/i)){
        file.launch();
        return true;
      }
      process.init(file);
      var arguments=this._getArguments(aArg);
      process.run(false, arguments, arguments.length);
      return true;

    }catch(e){ alert(e);return false;}
    return false;    // avoid JavaScript Error.
  },

  get clipboard(){
    try{
      var pastetext="";
      var clip =  Components.classes["@mozilla.org/widget/clipboard;1"]
            .createInstance(Components.interfaces.nsIClipboard);
      if (!clip) return "";
      var trans =  Components.classes["@mozilla.org/widget/transferable;1"]
            .createInstance(Components.interfaces.nsITransferable);
      if (!trans) return "";
      trans.addDataFlavor("text/unicode");

      clip.getData(trans,clip.kGlobalClipboard);

      var str=new Object();
      var strLength=new Object();

      trans.getTransferData("text/unicode",str,strLength);
      if (str) str=str.value.QueryInterface(Components.interfaces.nsISupportsString);
      if (str) pastetext=str.data.substring(0,strLength.value / 2);
      return pastetext;
    }catch(e){}
    return "";
  },

  get selected(){
    try{
      var selectedtext="";
      var focusedWindow = document.commandDispatcher.focusedWindow;
      if (!focusedWindow || focusedWindow == window)
        focusedWindow=window._content; 
      selectedtext = focusedWindow.getSelection().toString();
      return selectedtext;
    }catch(e){}
    return "";
  },

  get address(){
      var urlbar=document.getElementById("urlbar");
      return ( urlbar ) ? urlbar.value:"";
  },

  launch : function(aPath,aArg){
    if(! this._launch(aPath,aArg))
      this.settings();
  },

  invalidateWindows : function(){
    try{
      this._flush();
      var browserWindows=this._wm.getZOrderDOMWindowEnumerator('navigator:browser',true);
        while(browserWindows.hasMoreElements())
      {
        var browserWindow=browserWindows.getNext();
        if("ExternalAppRebuild" in browserWindow.top)
          browserWindow.top.setTimeout(ExternalAppRebuild,0);
       }
    }catch(e){ return;}
  },
  ///////////RDF//////////

  _rdfService  :  null,
  _ds  :  null,
  _container  :  null,

  _init: function(){

    //// INIT this.Datasource////////
    this._rdfService = Components.classes["@mozilla.org/rdf/rdf-service;1"]
      .getService(Components.interfaces.nsIRDFService);
    this._wm = Components.classes['@mozilla.org/appshell/window-mediator;1']
                  .getService(Components.interfaces.nsIWindowMediator);
    this._ds=this._rdfService.GetDataSourceBlocking( this._uri );
  
    //// INIT this.Container ///////
    this._container = Components.classes["@mozilla.org/rdf/container;1"]
      .getService(Components.interfaces.nsIRDFContainer);
    var RDFCUtils = Components.classes['@mozilla.org/rdf/container-utils;1']
      .getService(Components.interfaces.nsIRDFContainerUtils);
    var ContainerResource=this._rdfService.GetResource("urn:externalapp:buttons");

    if( RDFCUtils.IsContainer(this._ds,ContainerResource ) )
      this._container.Init(this._ds, ContainerResource);
    else this._container=RDFCUtils.MakeSeq(this._ds,ContainerResource);
  },

  clearData: function(){
    var resources=this._container.GetElements();
    var resource=null;
      while(resources.hasMoreElements()){
          resource=resources.getNext();
      resource=resource.QueryInterface(Components.interfaces.nsIRDFNode);
      this._container.RemoveElement(resource,false);
      this._unassert(this._ds,resource);
     }
    this.invalidateWindows();
  },

  appendData : function (){
    var fp =  Components.classes["@mozilla.org/filepicker;1"]
          .createInstance(Components.interfaces.nsIFilePicker);
    fp.init(window, 'External Application', Components.interfaces.nsIFilePicker.modeOpen);
    fp.appendFilters(Components.interfaces.nsIFilePicker.filterApps);
    fp.appendFilters(Components.interfaces.nsIFilePicker.filterAll);

    var ret = fp.show();
    if (ret != Components.interfaces.nsIFilePicker.returnOK) return;
    var file =  fp.fileURL.file.QueryInterface(Components.interfaces.nsILocalFile);
    this.appendDataFromFile(file)
  },

  _apendData: function(aResource,label,path,tooltiptext,ipath,isize,image,key,arguments){
    this._ds.Assert(aResource, this._arc("label"),  this._rdfService.GetLiteral(label), true);
    this._ds.Assert(aResource, this._arc("path"),  this._rdfService.GetLiteral(path), true);
    this._ds.Assert(aResource, this._arc("tooltiptext"),  this._rdfService.GetLiteral(tooltiptext), true);
    this._ds.Assert(aResource, this._arc("ipath"),  this._rdfService.GetLiteral(ipath), true);
    this._ds.Assert(aResource, this._arc("isize"),  this._rdfService.GetLiteral(isize), true);
    this._ds.Assert(aResource, this._arc("image"),  this._rdfService.GetLiteral(image), true);
    this._ds.Assert(aResource, this._arc("key"),  this._rdfService.GetLiteral(key), true);
    this._ds.Assert(aResource, this._arc("arguments"),  this._rdfService.GetLiteral(arguments), true);

  },

  _flush:function(){
    this._ds.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource)
        .Flush();
  },

  getProperty: function(aResource,aProp)
  {
    var node=this._ds.GetTarget(aResource,this._arc(aProp),true);
    if(node)
      return node.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
    else
      return "";
  },

  updateData: function(aURI,label,path,tooltiptext,ipath,isize,image,key,arguments){
    var resource=this._rdfService.GetResource(aURI);
    this._unassert(this._ds,resource);
    this._apendData(resource,label,path,tooltiptext,ipath,isize,image,key,arguments);
  },

  appendDataFromFile : function(aFile){
    var isize    = "?size="+ nsPreferences.getIntPref('externalapp.default.iconsize',16).toString();
    var prefix  = nsPreferences.copyUnicharPref('externalapp.default.tooltiptext_prefix',"Shortcut to ");
    var path    = aFile.path;
    var ipath   = aFile.path;
    var image   = "moz-icon:"+this.getURLFromFile(aFile)+isize;
    var label   = aFile.leafName;
    var tooltiptext  = prefix+label;
    if(label.match(/(.+)\.[a-zA-Z0-9]+$/))
      label=RegExp.$1;
    var itemResource=this._rdfService.GetResource("urn:externalapp:"+this.IDmaker());
    if (this._container.IndexOf(itemResource)==-1)
      this._container.AppendElement(itemResource);
    else
      this._unassert(this._ds,itemResource);
    this._apendData(itemResource,label,path,tooltiptext,ipath,isize,image,"","");
    this.invalidateWindows();
  },

  removeData : function(resid){
    var itemResource=this._rdfService.GetResource(resid);
    if(this._container.IndexOf(itemResource)<0) return;

    this._container.RemoveElement(itemResource,true);
    this._unassert(this._ds,itemResource);
    this.invalidateWindows();

  },

  move :function(FromURI,ToURI){
    var FromResource=this._rdfService.GetResource(FromURI);
    var ToResource=this._rdfService.GetResource(ToURI);
    var FromIndex=this._container.IndexOf(FromResource);
    var ToIndex=this._container.IndexOf(ToResource);
    
    if(FromIndex<0) return;
    if(ToIndex<0) return;

      this._container.RemoveElement(FromResource,true);
      this._container.InsertElementAt(FromResource,ToIndex,true);

    this.invalidateWindows();
  },

  //////Utilities////////////////
  _unassert: function (dsource,node)
  {
      var props=dsource.ArcLabelsOut(node);
      while(props.hasMoreElements()){
          var prop=props.getNext();
      var target=dsource.GetTarget(node,prop,true);
         dsource.Unassert(node,prop,target);
      }
  },

  IDmaker : function()
  {
    var d= new Date();
    return d.getTime().toString();
  }
};
